@@ -1,54 +1,52 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2013 by Dominique Dumont.
+# This software is Copyright (c) 2014 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Backend::Fstab ;
-{
-  $Config::Model::Backend::Fstab::VERSION = '2.047';
-}
-use Mouse ;
-use Carp ;
+package Config::Model::Backend::Fstab;
+$Config::Model::Backend::Fstab::VERSION = '2.061';
+use Mouse;
+use Carp;
 use Log::Log4perl qw(get_logger :levels);
- 
+
 extends 'Config::Model::Backend::Any';
 
-my $logger = get_logger("Backend::Fstab") ;
+my $logger = get_logger("Backend::Fstab");
 
-sub suffix { return '' ; }
+sub suffix { return ''; }
 
-sub annotation { return 1 ;}
+sub annotation { return 1; }
 
 my %opt_r_translate = (
-    ro => 'rw=0',
-    rw => 'rw=1',
-    bsddf => 'statfs_behavior=bsddf',
+    ro      => 'rw=0',
+    rw      => 'rw=1',
+    bsddf   => 'statfs_behavior=bsddf',
     minixdf => 'statfs_behavior=minixdf',
-) ;
+);
 
 sub read {
-    my $self = shift ;
-    my %args = @_ ;
+    my $self = shift;
+    my %args = @_;
 
     # args are:
     # root       => './my_test',  # fake root directory, userd for tests
-    # config_dir => /etc/foo',    # absolute path 
+    # config_dir => /etc/foo',    # absolute path
     # file       => 'foo.conf',   # file name
-    # file_path  => './my_test/etc/foo/foo.conf' 
+    # file_path  => './my_test/etc/foo/foo.conf'
     # io_handle  => $io           # IO::File object
     # check      => yes|no|skip
 
-    return 0 unless defined $args{io_handle} ; # no file to read
-    my $check = $args{check} || 'yes' ;
+    return 0 unless defined $args{io_handle};    # no file to read
+    my $check = $args{check} || 'yes';
 
-    my @lines = $args{io_handle}->getlines ;
+    my @lines = $args{io_handle}->getlines;
 
     # try to get global comments (comments before a blank line)
-    $self->read_global_comments(\@lines,'#') ;
+    $self->read_global_comments( \@lines, '#' );
 
     my @assoc = $self->associates_comments_with_data( \@lines, '#' );
     foreach my $item (@assoc) {
@@ -56,12 +54,13 @@ sub read {
         $logger->trace("fstab read data '$data' comment '$comment'");
 
         my ( $device, $mount_point, $type, $options, $dump, $pass ) =
-          split /\s+/, $data;
+            split /\s+/, $data;
 
         my $swap_idx = 0;
-        my $label = $device =~ /LABEL=(\w+)$/ ? $1
-          : $type eq 'swap' ? "swap-" . $swap_idx++
-          :                   $mount_point;
+        my $label =
+              $device =~ /LABEL=(\w+)$/ ? $1
+            : $type eq 'swap'           ? "swap-" . $swap_idx++
+            :                             $mount_point;
 
         my $fs_obj = $self->node->fetch_element('fs')->fetch_with_id($label);
 
@@ -71,7 +70,7 @@ sub read {
         }
 
         my $load_line = "fs_vfstype=$type fs_spec=$device fs_file=$mount_point "
-          . "fs_freq=$dump fs_passno=$pass";
+            . "fs_freq=$dump fs_passno=$pass";
         $logger->debug("Loading:$load_line\n");
         $fs_obj->load( step => $load_line, check => $check );
 
@@ -85,68 +84,69 @@ sub read {
         } @options;
 
         $logger->debug("Loading:@options");
-        $fs_obj->fetch_element('fs_mntopts')
-          ->load( step => "@options", check => $check );
+        $fs_obj->fetch_element('fs_mntopts')->load( step => "@options", check => $check );
     }
     return 1;
 }
 
 sub write {
-    my $self = shift ;
-    my %args = @_ ;
+    my $self = shift;
+    my %args = @_;
 
     # args are:
-    # object     => $obj,         # Config::Model::Node object 
+    # object     => $obj,         # Config::Model::Node object
     # root       => './my_test',  # fake root directory, userd for tests
-    # config_dir => /etc/foo',    # absolute path 
+    # config_dir => /etc/foo',    # absolute path
     # file       => 'foo.conf',   # file name
-    # file_path  => './my_test/etc/foo/foo.conf' 
+    # file_path  => './my_test/etc/foo/foo.conf'
     # io_handle  => $io           # IO::File object
     # check      => yes|no|skip
 
-    my $ioh = $args{io_handle} ;
-    my $node = $args{object} ;
+    my $ioh  = $args{io_handle};
+    my $node = $args{object};
 
     croak "Undefined file handle to write" unless defined $ioh;
 
-    $self->write_global_comment($ioh,'#') ;
+    $self->write_global_comment( $ioh, '#' );
 
     # Using Config::Model::ObjTreeScanner would be overkill
-    foreach my $line_obj ($node->fetch_element('fs')->fetch_all ) {
-        my $d = sprintf("%-30s %-25s %-6s %-10s %d %d\n",
-                     map ($line_obj->fetch_element_value($_), qw/fs_spec fs_file fs_vfstype/),
-                     $self->option_string($line_obj->fetch_element('fs_mntopts')) ,
-                     map ($line_obj->fetch_element_value($_) , qw/fs_freq fs_passno/),
-                    );
-        $self->write_data_and_comments($ioh,'#',$d, $line_obj->annotation) ;
-        
+    foreach my $line_obj ( $node->fetch_element('fs')->fetch_all ) {
+        my $d = sprintf(
+            "%-30s %-25s %-6s %-10s %d %d\n",
+            map ( $line_obj->fetch_element_value($_), qw/fs_spec fs_file fs_vfstype/ ),
+            $self->option_string( $line_obj->fetch_element('fs_mntopts') ),
+            map ( $line_obj->fetch_element_value($_), qw/fs_freq fs_passno/ ),
+        );
+        $self->write_data_and_comments( $ioh, '#', $d, $line_obj->annotation );
+
     }
 
     return 1;
 }
 
-my %rev_opt_r_translate = reverse %opt_r_translate ;
+my %rev_opt_r_translate = reverse %opt_r_translate;
 
 sub option_string {
-    my ($self,$obj) = @_ ;
-    
-    my @options ;
-    foreach my $opt ($obj->get_element_name ) {
-        my $v = $obj->fetch_element_value($opt) ;
-        next unless defined $v ;
-        my $key = "$opt=$v" ;
-        my $str = defined $rev_opt_r_translate{$key} ? $rev_opt_r_translate{$key} 
-                : "$v" eq '0'                        ? 'no'.$opt 
-                : "$v" eq '1'                        ? $opt 
-                :                                      $key ;
-        push @options , $str ;
+    my ( $self, $obj ) = @_;
+
+    my @options;
+    foreach my $opt ( $obj->get_element_name ) {
+        my $v = $obj->fetch_element_value($opt);
+        next unless defined $v;
+        my $key = "$opt=$v";
+        my $str =
+              defined $rev_opt_r_translate{$key} ? $rev_opt_r_translate{$key}
+            : "$v" eq '0'                        ? 'no' . $opt
+            : "$v" eq '1'                        ? $opt
+            :                                      $key;
+        push @options, $str;
     }
-    
-    return join',',@options ;
+
+    return join ',', @options;
 }
 
-no Mouse ;
-__PACKAGE__->meta->make_immutable ;
+no Mouse;
+__PACKAGE__->meta->make_immutable;
 
 1;
 
@@ -164,7 +164,7 @@ Config::Model::Backend::Fstab - Read and write config from fstab file
 
 =head1 VERSION
 
-version 2.047
+version 2.061
 
 =head1 SYNOPSIS
 
@@ -227,7 +227,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2013 by Dominique Dumont.
+This software is Copyright (c) 2014 by Dominique Dumont.
 
 This is free software, licensed under:
 
