@@ -1,38 +1,33 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2013 by Dominique Dumont.
+# This software is Copyright (c) 2014 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::BackendMgr ;
-{
-  $Config::Model::BackendMgr::VERSION = '2.047';
-}
-
-use Mouse ;
-use namespace::autoclean;
+package Config::Model::BackendMgr;
+$Config::Model::BackendMgr::VERSION = '2.061';
+use Mouse;
 
 use Carp;
 
-use Config::Model::Exception ;
-use Data::Dumper ;
-use File::Path ;
-use File::Copy ;
-use File::HomeDir ;
-use IO::File ;
-use Storable qw/dclone/ ;
-use Scalar::Util qw/weaken/ ;
+use Config::Model::Exception;
+use Data::Dumper;
+use File::Path;
+use File::Copy;
+use File::HomeDir;
+use IO::File;
+use Storable qw/dclone/;
+use Scalar::Util qw/weaken/;
 use Log::Log4perl qw(get_logger :levels);
 
-my $logger = get_logger('BackendMgr') ;
+my $logger = get_logger('BackendMgr');
 
 # used only for tests
 my $__test_home = '';
-sub _set_test_home { $__test_home = shift ;}
-
+sub _set_test_home { $__test_home = shift; }
 
 # one BackendMgr per file
 
@@ -46,120 +41,139 @@ has 'file_backup' => ( is => 'rw', isa => 'ArrayRef', default => sub { [] } );
 has 'backend' => (
     is      => 'rw',
     isa     => 'HashRef[Config::Model::Backend::Any]',
-    traits    => ['Hash'],
+    traits  => ['Hash'],
     default => sub { {} },
-    handles => { set_backend => 'set', get_backend => 'get' }
-);
+    handles => { set_backend => 'set', get_backend => 'get' } );
 
 #
 # New subroutine "get_cfg_dir_path" extracted - Thu Jul 11 13:47:22 2013.
 #
 sub get_cfg_dir_path {
-    my $self = shift ;
+    my $self = shift;
     my %args = @_;
 
-    my $w = $args{write} || 0 ;
-    my $dir = $args{os_config_dir}{$^O} || $args{config_dir} ;
-    if ($dir =~ /^~/) { 
+    my $w = $args{write} || 0;
+    my $dir = $args{os_config_dir}{$^O} || $args{config_dir};
+    if ( $dir =~ /^~/ ) {
         my $home = $__test_home || File::HomeDir->my_home;
         $dir =~ s/^~/$home/;
     }
-    
-    $dir = $args{root}.$dir ;
-    
-    $dir .= '/' unless $dir =~ m!/$! ;
-    if (not -d $dir and $w and $args{auto_create}) {
-        $logger->info("creating directory $dir" );
-        mkpath ($dir,0, 0755);
+
+    $dir = $args{root} . $dir;
+
+    $dir .= '/' unless $dir =~ m!/$!;
+    if ( not -d $dir and $w and $args{auto_create} ) {
+        $logger->info("creating directory $dir");
+        mkpath( $dir, 0, 0755 );
     }
 
-    unless (-d $dir) { 
-        $logger->info( "auto_". ($w ? 'write' : 'read')
-                      ." $args{backend} no directory $dir" );
-        return (0, $dir);
+    unless ( -d $dir ) {
+        $logger->info( "auto_" . ( $w ? 'write' : 'read' ) . " $args{backend} no directory $dir" );
+        return ( 0, $dir );
     }
 
-    $logger->debug("dir: ". $dir // '<undef>') ;
+    $logger->debug( "dir: " . $dir // '<undef>' );
 
-    return (1, $dir);
+    return ( 1, $dir );
 }
 
 sub get_cfg_file_path {
-    my $self = shift ;
+    my $self = shift;
     my %args = @_;
 
-    my $w = $args{write} || 0 ;
+    my $w = $args{write} || 0;
 
     # config file override
-    if (defined $args{config_file}) {
-        my $override = $args{root}.$args{config_file}  ;
-         $logger->trace("auto_". ($w ? 'write' : 'read')
-                  ." $args{backend} override target file is $override" );
-        return (1, $args{root}.$args{config_file});
+    my $cfo = $args{config_file};
+
+    if ( defined $cfo and $cfo eq '-' and $w == 0 ) {
+        $logger->trace("auto_read: $args{backend} override target file is STDIN");
+        return ( 1, '-' );
     }
 
-    Config::Model::Exception::Model -> throw (
-         error=> "backend error: empty 'config_dir' parameter (and no config_file override)",
-         object => $self->node
+    if ( defined $cfo ) {
+        my $override = $args{root} . $args{config_file};
+        $logger->trace( "auto_"
+                . ( $w ? 'write' : 'read' )
+                . " $args{backend} override target file is $override" );
+        return ( 1, $args{root} . $args{config_file} );
+    }
+
+    Config::Model::Exception::Model->throw(
+        error  => "backend error: empty 'config_dir' parameter (and no config_file override)",
+        object => $self->node
     ) unless $args{config_dir};
 
-    my ($dir_ok,$dir) = $self->get_cfg_dir_path (%args);
+    my ( $dir_ok, $dir ) = $self->get_cfg_dir_path(%args);
 
-    if (defined $args{file}) {
-        my $res = $dir.$args{file} ;
+    if ( defined $args{file} ) {
+        my $res = $dir . $args{file};
         $logger->trace("get_cfg_file_path: returns $res");
-        return ($dir_ok, $res) ;
+        return ( $dir_ok, $res );
     }
 
-    if (not defined $args{suffix}) {
+    if ( not defined $args{suffix} ) {
         $logger->trace("get_cfg_file_path: returns undef (no suffix, no file argument)");
         return (0);
     }
 
-    my $i = $self->node->instance ;
-    my $name = $dir. $i->name ;
+    my $i    = $self->node->instance;
+    my $name = $dir . $i->name;
 
     # append ":foo bar" if not root object
-    my $loc = $self->node->location ; # not very good
+    my $loc = $self->node->location;    # not very good
     if ($loc) {
-        if (($w and not -d $name and $args{auto_create})) {
-          $logger->info("get_cfg_file_path: auto_write create subdirectory ",
-                        "$name (location $loc)" );
-          mkpath ($name,0, 0755);
+        if ( ( $w and not -d $name and $args{auto_create} ) ) {
+            $logger->info( "get_cfg_file_path: auto_write create subdirectory ",
+                "$name (location $loc)" );
+            mkpath( $name, 0, 0755 );
         }
-        $name .= '/'.$loc ;
+        $name .= '/' . $loc;
     }
 
-    $name .= $args{suffix} ;
+    $name .= $args{suffix};
 
-    $logger->trace("get_cfg_file_path: auto_". ($w ? 'write' : 'read')
-                  ." $args{backend} target file is $name" );
+    $logger->trace( "get_cfg_file_path: auto_"
+            . ( $w ? 'write' : 'read' )
+            . " $args{backend} target file is $name" );
 
-    return (1,$name);
+    return ( 1, $name );
 }
 
 sub open_read_file {
-    my $self = shift ;
-    my %args = @_ ;
+    my $self = shift;
+    my %args = @_;
 
-    my ($file_ok,$file_path) = $self->get_cfg_file_path(%args);
+    my ( $file_ok, $file_path ) = $self->get_cfg_file_path(%args);
+
+    if ( $file_ok and $file_path eq '-' ) {
+        my $io = IO::Handle->new();
+        if ( $io->fdopen( fileno(STDIN), "r" ) ) {
+            return ( 1, '-', $io );
+        }
+        else {
+            return ( 0, '-' );
+        }
+    }
 
     # not very clean
-    return (0, $file_path) if $args{backend} =~ /_file$/
-        and (not $file_ok or not -r $file_path) ;
+    return ( 0, $file_path )
+        if $args{backend} =~ /_file$/
+        and ( not $file_ok or not -r $file_path );
 
     my $fh = new IO::File;
-    if ($file_ok and -e $file_path) {
+    if ( $file_ok and -e $file_path ) {
         $logger->debug("open_read_file: open $file_path for read");
         $fh->open($file_path);
         $fh->binmode(":utf8");
+
         # store a backup in memory in case there's a problem
-        $self->file_backup( [ $fh-> getlines ] ) ;
-        $fh->seek(0,0) ; # go back to beginning of file
-        return (1,$file_path,$fh) ;
+        $self->file_backup( [ $fh->getlines ] );
+        $fh->seek( 0, 0 );    # go back to beginning of file
+        return ( 1, $file_path, $fh );
     }
     else {
-        return (0,$file_path) ;
+        return ( 0, $file_path );
     }
 }
 
@@ -168,160 +182,163 @@ sub open_read_file {
 # New subroutine "load_backend_class" extracted - Thu Aug 12 18:32:37 2010.
 #
 sub load_backend_class {
-    my $backend = shift;
-    my $function = shift ;
+    my $backend  = shift;
+    my $function = shift;
 
     $logger->debug("load_backend_class: called with backend $backend, function $function");
-    my %c ;
+    my %c;
 
-    my $k = "Config::Model::Backend::".ucfirst($backend) ;
-    my $f = $k.'.pm';
+    my $k = "Config::Model::Backend::" . ucfirst($backend);
+    my $f = $k . '.pm';
     $f =~ s!::!/!g;
-    $c{$k} = $f ;
-    
+    $c{$k} = $f;
+
     # try another class
     $k =~ s/_(\w)/uc($1)/ge;
     $f =~ s/_(\w)/uc($1)/ge;
-    $c{$k} = $f ;
+    $c{$k} = $f;
 
-    foreach my $c (keys %c) { 
-        if ($c->can($function)) {
-            # no need to load class  
+    foreach my $c ( keys %c ) {
+        if ( $c->can($function) ) {
+
+            # no need to load class
             $logger->debug("load_backend_class: $c is already loaded (can $function)");
-            return $c ;
-        } 
-    }
-
-        
-    # look for file to load 
-    my $class_to_load ;
-    foreach my $c (keys %c) { 
-        $logger->debug("load_backend_class: looking to load class $c");
-        foreach my $prefix (@INC) {
-            my $realfilename = "$prefix/$c{$c}";
-            $class_to_load = $c if -f $realfilename ;
+            return $c;
         }
     }
 
-    return unless defined $class_to_load ;
-    my $file_to_load = $c{$class_to_load} ;
+    # look for file to load
+    my $class_to_load;
+    foreach my $c ( keys %c ) {
+        $logger->debug("load_backend_class: looking to load class $c");
+        foreach my $prefix (@INC) {
+            my $realfilename = "$prefix/$c{$c}";
+            $class_to_load = $c if -f $realfilename;
+        }
+    }
+
+    return unless defined $class_to_load;
+    my $file_to_load = $c{$class_to_load};
 
     $logger->debug("load_backend_class: loading class $class_to_load, $file_to_load");
-    eval {require $file_to_load; } ;
+    eval { require $file_to_load; };
 
     if ($@) {
-            die "Could not parse $file_to_load: $@\n";
-    } 
-    return $class_to_load ;
+        die "Could not parse $file_to_load: $@\n";
+    }
+    return $class_to_load;
 }
 
 sub read_config_data {
-    my ($self, %args) = @_ ;
+    my ( $self, %args ) = @_;
 
     {
         no warnings 'uninitialized';
-        $logger->debug("called with ",join(' ',%args));
+        $logger->debug( "called with ", join( ' ', %args ) );
     }
-    
-    my $readlist_orig = delete $args{read_config} ;
-    my $check = delete $args{check} ;
-    my $r_dir = delete $args{read_config_dir} ;
-    my $config_file_override = delete $args{config_file} ;
-    my $auto_create_override = delete $args{auto_create} ;
-    
-    croak "unexpected args ".join(' ',keys %args)."\n" if %args ;
 
+    my $readlist_orig        = delete $args{read_config};
+    my $check                = delete $args{check};
+    my $r_dir                = delete $args{read_config_dir};
+    my $config_file_override = delete $args{config_file};
+    my $auto_create_override = delete $args{auto_create};
+
+    croak "unexpected args " . join( ' ', keys %args ) . "\n" if %args;
 
     # r_dir is obsolete
-    if (defined $r_dir) {
-        die $self->node->config_class_name," : read_config_dir is obsolete\n";
+    if ( defined $r_dir ) {
+        die $self->node->config_class_name, " : read_config_dir is obsolete\n";
     }
 
     my $readlist = dclone $readlist_orig ;
 
-    my $instance = $self->node->instance() ;
+    my $instance = $self->node->instance();
 
     # root override is passed by the instance
-    my $root_dir = $instance -> read_root_dir || '';
-    $root_dir .= '/' if ($root_dir and $root_dir !~ m(/$)) ; 
+    my $root_dir = $instance->read_root_dir || '';
+    $root_dir .= '/' if ( $root_dir and $root_dir !~ m(/$) );
 
-    croak "readlist must be array or hash ref\n" 
-      unless ref $readlist ;
+    croak "readlist must be array or hash ref\n"
+        unless ref $readlist;
 
-    my @list = ref $readlist  eq 'ARRAY' ? @$readlist :  ($readlist) ;
-    my $pref_backend = $instance->backend || '' ;
-    my $read_done = 0;
-    my $auto_create = 0;
+    my @list = ref $readlist eq 'ARRAY' ? @$readlist : ($readlist);
+    my $pref_backend = $instance->backend || '';
+    my $read_done    = 0;
+    my $auto_create  = 0;
     my @tried;
 
     foreach my $read (@list) {
-        warn $self->config_class_name,
-          " deprecated 'syntax' parameter in backend\n" if defined $read->{syntax} ;
+        warn $self->config_class_name, " deprecated 'syntax' parameter in backend\n"
+            if defined $read->{syntax};
         my $backend = delete $read->{backend} || delete $read->{syntax} || 'custom';
-        if ($backend =~ /^(perl|ini|cds)$/) {
+        if ( $backend =~ /^(perl|ini|cds)$/ ) {
             warn $self->config_class_name,
-              " deprecated  backend $backend. Should be '$ {backend}_file'\n";
-            $backend .= "_file" ;
+                " deprecated  backend $backend. Should be '$ {backend}_file'\n";
+            $backend .= "_file";
         }
 
-        next if ($pref_backend and $backend ne $pref_backend) ;
+        next if ( $pref_backend and $backend ne $pref_backend );
 
-        if (defined $read->{allow_empty}) {
+        if ( defined $read->{allow_empty} ) {
             warn "backend $backend: allow_empty is deprecated. Use auto_create";
-            $auto_create ||= delete $read->{allow_empty} ;
+            $auto_create ||= delete $read->{allow_empty};
         }
 
         $auto_create ||= delete $read->{auto_create} if defined $read->{auto_create};
 
-        if ($read->{default_layer}) {
-            $self->read_config_sub_layer($read, $root_dir, $config_file_override, $check, $backend);
+        if ( $read->{default_layer} ) {
+            $self->read_config_sub_layer( $read, $root_dir, $config_file_override, $check,
+                $backend );
         }
 
-        my ($res, $file) = $self->try_read_backend ($read, $root_dir, $config_file_override, $check, $backend);
+        my ( $res, $file ) =
+            $self->try_read_backend( $read, $root_dir, $config_file_override, $check, $backend );
         push @tried, $file;
 
-        if ($res) { 
-            $read_done = 1 ;
+        if ($res) {
+            $read_done = 1;
             last;
         }
     }
 
-    Config::Model::Exception::ConfigFile::Missing -> throw (
-         tried_files => \@tried,
-         object => $self->node,
-    ) unless $read_done or $auto_create_override or $auto_create;
+    Config::Model::Exception::ConfigFile::Missing->throw(
+        tried_files => \@tried,
+        object      => $self->node,
+        )
+        unless $read_done
+        or $auto_create_override
+        or $auto_create;
 
 }
 
-
 sub read_config_sub_layer {
-    my ($self, $read, $root_dir, $config_file_override, $check, $backend) = @_;
+    my ( $self, $read, $root_dir, $config_file_override, $check, $backend ) = @_;
 
-    my $layered_config = delete $read->{default_layer} ;
-    my $layered_read = dclone $read ;
+    my $layered_config = delete $read->{default_layer};
+    my $layered_read   = dclone $read ;
 
-    map { my $lc = delete $layered_config->{$_} ; $layered_read->{$_} = $lc if $lc ;}
-        qw/file config_dir os_config_dir/ ;
+    map { my $lc = delete $layered_config->{$_}; $layered_read->{$_} = $lc if $lc; }
+        qw/file config_dir os_config_dir/;
 
     Config::Model::Exception::Model->throw(
         error => "backend error: unexpected default_layer parameters: "
-          . join( ' ', keys %$layered_config ),
+            . join( ' ', keys %$layered_config ),
         object => $self->node,
     ) if %$layered_config;
 
-    my $i = $self->node->instance ;
-    my $already_in_layered = $i->layered ;
+    my $i                  = $self->node->instance;
+    my $already_in_layered = $i->layered;
 
     # layered stuff here
-    if (not $already_in_layered) {
-        $i->layered_clear ;
-        $i->layered_start ;
+    if ( not $already_in_layered ) {
+        $i->layered_clear;
+        $i->layered_start;
     }
 
-    $self->try_read_backend ($layered_read, $root_dir, $config_file_override, $check, $backend);
+    $self->try_read_backend( $layered_read, $root_dir, $config_file_override, $check, $backend );
 
-    if (not $already_in_layered) {
-        $i->layered_stop ;
+    if ( not $already_in_layered ) {
+        $i->layered_stop;
     }
 }
 
@@ -372,12 +389,12 @@ sub try_read_backend {
     }
     elsif ( $backend eq 'perl_file' ) {
         ( $file_ok, $file_path, $fh ) = $self->open_read_file( @read_args, suffix => '.pl' );
-        return (0, $file_path) unless $file_ok;
+        return ( 0, $file_path ) unless $file_ok;
         eval { $res = $self->read_perl( @read_args, file_path => $file_path, io_handle => $fh ); };
     }
     elsif ( $backend eq 'cds_file' ) {
         ( $file_ok, $file_path, $fh ) = $self->open_read_file( @read_args, suffix => '.cds' );
-        return (0, $file_path) unless $file_ok;
+        return ( 0, $file_path ) unless $file_ok;
         eval {
             $res = $self->read_cds_file(
                 @read_args,
@@ -389,8 +406,8 @@ sub try_read_backend {
     else {
         # try to load a specific Backend class
         my $f = delete $read->{function} || 'read';
-        my $c = load_backend_class( $backend, $f ) ;
-        return (0,'unknown') unless defined $c;
+        my $c = load_backend_class( $backend, $f );
+        return ( 0, 'unknown' ) unless defined $c;
 
         no strict 'refs';
         my $backend_obj = $c->new( node => $self->node, name => $backend );
@@ -398,7 +415,7 @@ sub try_read_backend {
         my $suffix;
         $suffix = $backend_obj->suffix if $backend_obj->can('suffix');
         ( $file_ok, $file_path, $fh ) = $self->open_read_file( @read_args, suffix => $suffix );
-        $logger->info( "Read with $backend " . $c . "::$f on $file_path"  );
+        $logger->info( "Read with $backend " . $c . "::$f on $file_path" );
 
         eval {
             $res = $backend_obj->$f(
@@ -422,292 +439,315 @@ sub try_read_backend {
         ref $e ? $e->rethrow : die $e;
     }
 
-    return ($res, $file_path);
+    return ( $res, $file_path );
 }
 
 sub auto_write_init {
-    my ($self, %args) = @_ ;
-    my $wrlist_orig = delete $args{write_config} ;
-    my $w_dir = delete $args{write_config_dir} ;
+    my ( $self, %args ) = @_;
+    my $wrlist_orig = delete $args{write_config};
+    my $w_dir       = delete $args{write_config_dir};
 
-    weaken($self) ; # avoid leak: $self is stored in write_back closure
+    weaken($self);    # avoid leak: $self is stored in write_back closure
 
-    croak "auto_write_init: unexpected args ".join(' ',keys %args)."\n" 
-        if %args ;
+    croak "auto_write_init: unexpected args " . join( ' ', keys %args ) . "\n"
+        if %args;
 
     # w_dir is obsolete
-    if (defined $w_dir) {
-        die $self->config_class_name," : write_config_dir is obsolete\n";
+    if ( defined $w_dir ) {
+        die $self->config_class_name, " : write_config_dir is obsolete\n";
     }
 
     my $wrlist = dclone $wrlist_orig ;
 
-    my $instance = $self->node->instance() ;
+    my $instance = $self->node->instance();
 
     # root override is passed by the instance
-    my $root_dir = $instance -> write_root_dir || '';
+    my $root_dir = $instance->write_root_dir || '';
 
-    my @array = ref $wrlist eq 'ARRAY' ? @$wrlist : ($wrlist) ;
+    my @array = ref $wrlist eq 'ARRAY' ? @$wrlist : ($wrlist);
 
     # ensure that one auto_create specified applies to all wr backends
     my $auto_create = 0;
     foreach my $write (@array) {
-        $auto_create ||= delete $write->{auto_create} 
-          if defined $write->{auto_create};
+        $auto_create ||= delete $write->{auto_create}
+            if defined $write->{auto_create};
     }
 
     # provide a proper write back function
     foreach my $write (@array) {
-        warn $self->config_class_name,
-          " deprecated 'syntax' parameter in auto_write\n" if defined $write->{syntax} ;
+        warn $self->config_class_name, " deprecated 'syntax' parameter in auto_write\n"
+            if defined $write->{syntax};
         my $backend = delete $write->{backend} || delete $write->{syntax} || 'custom';
-        if ($backend =~ /^(perl|ini|cds)$/) {
+        if ( $backend =~ /^(perl|ini|cds)$/ ) {
             warn $self->config_class_name,
-              " deprecated backend $backend. Should be '$ {backend}_file'\n";
-            $backend .= "_file" ;
+                " deprecated backend $backend. Should be '$ {backend}_file'\n";
+            $backend .= "_file";
         }
 
-        my $write_dir =  $write->{os_config_dir}{$^O} || $write->{config_dir} || '';
-        $write_dir .= '/' if $write_dir and $write_dir !~ m(/$) ; 
+        my $write_dir = $write->{os_config_dir}{$^O} || $write->{config_dir} || '';
+        $write_dir .= '/' if $write_dir and $write_dir !~ m(/$);
 
-        my ($fh, $file_ok) ;
-        $fh = new IO::File ; # opened in write callback
+        $logger->debug( "auto_write_init creating write cb ($backend) for ", $self->node->name );
 
-        $logger->debug("auto_write_init creating write cb ($backend) for ",$self->node->name);
+        my @wr_args = (
+            %$write,    # model data
+            auto_create => $auto_create,
+            backend     => $backend,
+            config_dir  => $write_dir,     # override from instance
+            write       => 1,              # for get_cfg_file_path
+            root        => $root_dir,      # override from instance
+        );
 
-        my @wr_args = (%$write,                  # model data
-                       auto_create => $auto_create,
-                       backend     => $backend,
-                       config_dir  => $write_dir, # override from instance
-                       io_handle   => $fh,
-                       write       => 1,          # for get_cfg_file_path
-                       root        => $root_dir,  # override from instance
-                      );
-
-        my $wb ;
-        if ($backend eq 'custom') {
-            my $c = my $file = $write->{class} ;
+        my $wb;
+        if ( $backend eq 'custom' ) {
+            my $c = my $file = $write->{class};
             $file =~ s!::!/!g;
-            my $f = $write->{function} || 'write' ;
-            require $file.'.pm' unless $c->can($f) ;
+            my $f = $write->{function} || 'write';
+            require $file . '.pm' unless $c->can($f);
 
-            $wb = sub  {  
+            $wb = sub {
                 no strict 'refs';
-                my $file_path ;
-                $logger->debug("write cb ($backend) called for ",$self->node->name);
-                ($file_ok, $file_path) = $self-> open_file_to_write($backend,$fh,@wr_args,@_)
-                    unless ($c->can('skip_open') and $c->skip_open) ;
-                my $res ;
+                $logger->debug( "write cb ($backend) called for ", $self->node->name );
+                my ( $file_ok, $file_path, $fh ) =
+                    $self->open_file_to_write( $backend, @wr_args, @_ )
+                    unless ( $c->can('skip_open') and $c->skip_open );
+                my $res;
                 $res = eval {
+
                     # override needed for "save as" button
-                    &{$c.'::'.$f}(@wr_args,
-                                  file_path => $file_path,
-                                  conf_dir => $write_dir, # legacy FIXME
-                                  object => $self->node, 
-                                  @_                      # override from user
-                                ) ;
+                    &{ $c . '::' . $f }(
+                        @wr_args,
+                        io_handle => $fh,
+                        file_path => $file_path,
+                        conf_dir  => $write_dir,    # legacy FIXME
+                        object    => $self->node,
+                        @_                          # override from user
+                    );
                 };
-                $logger->warn("write backend $c".'::'."$f failed: $@") if $@;
-                $self->close_file_to_write($@,$fh,$file_path) ;
-                return defined $res ? $res : $@ ? 0 : 1 ;
-             };
-            $self->{auto_write}{custom} = 1 ;
+                $logger->warn( "write backend $c" . '::' . "$f failed: $@" ) if $@;
+                $self->close_file_to_write( $@, $fh, $file_path );
+                return defined $res ? $res : $@ ? 0 : 1;
+            };
+            $self->{auto_write}{custom} = 1;
         }
-        elsif ($backend eq 'perl_file') {
+        elsif ( $backend eq 'perl_file' ) {
             $wb = sub {
-                $logger->debug("write cb ($backend) called for ",$self->node->name);
-                my ($file_ok, $file_path)
-                    = $self-> open_file_to_write($backend,$fh,
-                                                suffix => '.pl',@wr_args,@_) ;
-                my $res ;
+                $logger->debug( "write cb ($backend) called for ", $self->node->name );
+                my ( $file_ok, $file_path, $fh ) =
+                    $self->open_file_to_write( $backend, suffix => '.pl', @wr_args, @_ );
+                my $res;
                 $res = eval {
-                    $self->write_perl(@wr_args, file_path => $file_path,  @_) ;
+                    $self->write_perl( @wr_args, io_handle => $fh, file_path => $file_path, @_ );
                 };
-                $self->close_file_to_write($@,$fh,$file_path) ;
+                $self->close_file_to_write( $@, $fh, $file_path );
                 $logger->warn("write backend $backend failed: $@") if $@;
-                return defined $res ? $res : $@ ? 0 : 1 ;
-            } ;
-            $self->{auto_write}{perl_file} = 1 ;
+                return defined $res ? $res : $@ ? 0 : 1;
+            };
+            $self->{auto_write}{perl_file} = 1;
         }
-        elsif ($backend eq 'cds_file') {
+        elsif ( $backend eq 'cds_file' ) {
             $wb = sub {
-                $logger->debug("write cb ($backend) called for ",$self->node->name);
-                my ($file_ok, $file_path)
-                   = $self-> open_file_to_write($backend,$fh,
-                                                suffix => '.cds',@wr_args,@_) ;
-                my $res ;
+                $logger->debug( "write cb ($backend) called for ", $self->node->name );
+                my ( $file_ok, $file_path, $fh ) =
+                    $self->open_file_to_write( $backend, suffix => '.cds', @wr_args, @_ );
+                my $res;
                 $res = eval {
-                    $self->write_cds_file(@wr_args, file_path => $file_path, @_) ;
+                    $self->write_cds_file(
+                        @wr_args,
+                        io_handle => $fh,
+                        file_path => $file_path,
+                        @_
+                    );
                 };
                 $logger->warn("write backend $backend failed: $@") if $@;
-                $self->close_file_to_write($@,$fh,$file_path) ;
-                return defined $res ? $res : $@ ? 0 : 1 ;
-            } ;
-            $self->{auto_write}{cds_file} = 1 ;
+                $self->close_file_to_write( $@, $fh, $file_path );
+                return defined $res ? $res : $@ ? 0 : 1;
+            };
+            $self->{auto_write}{cds_file} = 1;
         }
         else {
-            my $f = $write->{function} || 'write' ;
-            my $c = load_backend_class ($backend, $f);
+            my $f = $write->{function} || 'write';
+            my $c = load_backend_class( $backend, $f );
 
             $wb = sub {
                 no strict 'refs';
-                $logger->debug("write cb ($backend) called for ",$self->node->name);
-                my $backend_obj =  $self->get_backend($backend)
-                                || $c->new(node => $self->node, name => $backend) ;
-                my $file_path ;
-                my $suffix ;
-                $suffix = $backend_obj->suffix if $backend_obj->can('suffix');
-                ($file_ok, $file_path)
-                    = $self-> open_file_to_write($backend,$fh, suffix => $suffix, @wr_args,@_)
-                    unless ($c->can('skip_open') and $c->skip_open) ;
-                my $res ;
-                $res = eval {
+                $logger->debug( "write cb ($backend) called for ", $self->node->name );
+                my $backend_obj = $self->get_backend($backend)
+                    || $c->new( node => $self->node, name => $backend );
+                my $suffix = $backend_obj->suffix if $backend_obj->can('suffix');
+                my ( $file_ok, $file_path, $fh ) =
+                    $self->open_file_to_write( $backend, suffix => $suffix, @wr_args, @_ )
+                    unless ( $c->can('skip_open') and $c->skip_open );
+                my $res = eval {
+
                     # override needed for "save as" button
-                    $backend_obj->$f( @wr_args, 
-                                      file_path => $file_path,
-                                      object => $self->node, 
-                                      @_                      # override from user
-                                    ) ;
-                } ;
-                $logger->warn("write backend $backend $c".'::'."$f failed: $@") if $@;
-                $self->close_file_to_write($@,$fh,$file_path) ;
-                return defined $res ? $res : $@ ? 0 : 1 ;
-             };
+                    $backend_obj->$f(
+                        @wr_args,
+                        io_handle => $fh,
+                        file_path => $file_path,
+                        object    => $self->node,
+                        @_    # override from user
+                    );
+                };
+                $logger->warn( "write backend $backend $c" . '::' . "$f failed: $@" ) if $@;
+                $self->close_file_to_write( $@, $fh, $file_path );
+                return defined $res ? $res : $@ ? 0 : 1;
+            };
         }
 
         # FIXME: enhance write back mechanism so that different backend *and* different nodse
         # work as expected
-        $logger->debug("registering write $backend in node ".$self->node->name);
-        push @{$self->{write_back}}, [$backend, $wb] ;
-        $instance->register_write_back($self->node->location) ;
+        $logger->debug( "registering write $backend in node " . $self->node->name );
+        push @{ $self->{write_back} }, [ $backend, $wb ];
+        $instance->register_write_back( $self->node->location );
     }
 }
 
 sub write_back {
-    my $self = shift ;
-    my %args = @_ ; 
+    my $self = shift;
+    my %args = @_;
 
-    my $force_backend = delete $args{backend} || '' ;
+    my $force_backend = delete $args{backend} || '';
 
-    croak "write_back: no subs registered in node", $self->node->location,". cannot save data\n" 
-        unless @{$self->{write_back}} ;
+    croak "write_back: no subs registered in node", $self->node->location, ". cannot save data\n"
+        unless @{ $self->{write_back} };
 
-    my @backends = @{$self->{write_back}} ;
-    $logger->debug("write_back called on node '",$self->node->name, "' for " , scalar @backends, " backends");
+    my @backends = @{ $self->{write_back} };
+    $logger->debug(
+        "write_back called on node '",
+        $self->node->name, "' for ", scalar @backends,
+        " backends"
+    );
 
-
-    my $dir = $args{config_dir} ;
-    mkpath($dir,0,0755) if $dir and not -d $dir ;
+    my $dir = $args{config_dir};
+    mkpath( $dir, 0, 0755 ) if $dir and not -d $dir;
 
     foreach my $wb_info (@backends) {
-	my ($backend,$wb) = @$wb_info ;
-	if (not $force_backend 
-	    or  $force_backend eq $backend 
-	    or  $force_backend eq 'all' ) {
-	    # exit when write is successfull
-	    my $res = $wb->(%args) ; 
-	    $logger->info("write_back called with $backend backend, result is ", defined $res ? $res : '<undef>' );
-	    last if ($res and not $force_backend); 
-	}
+        my ( $backend, $wb ) = @$wb_info;
+        if (   not $force_backend
+            or $force_backend eq $backend
+            or $force_backend eq 'all' ) {
+
+            # exit when write is successfull
+            my $res = $wb->(%args);
+            $logger->info( "write_back called with $backend backend, result is ",
+                defined $res ? $res : '<undef>' );
+            last if ( $res and not $force_backend );
+        }
     }
-    $logger->debug("write_back on node '",$self->node->name, "' done");
+    $logger->debug( "write_back on node '", $self->node->name, "' done" );
 }
 
 sub open_file_to_write {
-    my ($self, $backend, $fh, %args) = @_ ;
+    my ( $self, $backend, %args ) = @_;
 
-    my $backup = delete $args{backup};
+    my $backup    = delete $args{backup};
     my $do_backup = defined $backup;
-    $backup ||= 'old'; # use old only if defined
-    $backup = '.'.$backup unless $backup =~ /^\./;
+    $backup ||= 'old';    # use old only if defined
+    $backup = '.' . $backup unless $backup =~ /^\./;
 
-    my ($file_ok, $file_path) = $self->get_cfg_file_path(%args);
-    if ($file_ok) {
-        if ($do_backup and -r $file_path) {
-            copy($file_path, $file_path.$backup) or die "Backup copy failed: $!";
+    my ( $file_ok, $file_path ) = $self->get_cfg_file_path(%args);
+
+    if ( $file_ok and $file_path eq '-' ) {
+        my $io = IO::Handle->new();
+        if ( $io->fdopen( fileno(STDOUT), "w" ) ) {
+            return ( 1, '-', $io );
+        }
+        else {
+            return ( 0, '-' );
+        }
+    }
+    elsif ($file_ok) {
+        if ( $do_backup and -r $file_path ) {
+            copy( $file_path, $file_path . $backup ) or die "Backup copy failed: $!";
         }
         $logger->debug("$backend backend opened file $file_path to write");
-        $fh ->open("> $file_path") || die "Cannot open $file_path:$!";
+        my $fh = new IO::File;
+        $fh->open("> $file_path") || die "Cannot open $file_path:$!";
         $fh->binmode(':utf8');
+        return ( $file_ok, $file_path, $fh );
+    }
+    else {
+        return ( 0, $file_path );
     }
-
-    return ($file_ok, $file_path) ;
 }
 
 sub close_file_to_write {
-    my ($self,$error,$fh,$file_path) = @_ ;
-    
-    return unless defined $file_path ;
-    
+    my ( $self, $error, $fh, $file_path ) = @_;
+
+    return unless defined $file_path;
+
     if ($error) {
+
         # restore backup and display error
-        my $data = $self->file_backup ;
-        $logger->debug("Error during write, restoring backup in $file_path with ".scalar @$data." lines");
-        $fh->seek(0,0) ; # go back to beginning of file
+        my $data = $self->file_backup;
+        $logger->debug(
+            "Error during write, restoring backup in $file_path with " . scalar @$data . " lines" );
+        $fh->seek( 0, 0 );    # go back to beginning of file
         $fh->print(@$data);
         $fh->close;
-        $error->rethrow if ref ($error);
-        die $error ;
+        $error->rethrow if ref($error);
+        die $error;
     }
 
     $fh->close;
-    
+
     # check file size and remove empty files
-    unlink($file_path) if -z $file_path ;
+    unlink($file_path) if -z $file_path;
 }
 
 sub is_auto_write_for_type {
     my $self = shift;
-    my $type = shift ;
+    my $type = shift;
     return $self->{auto_write}{$type} || 0;
 }
 
 sub read_cds_file {
     my $self = shift;
-    my %args = @_ ;
+    my %args = @_;
 
-    my $file_path = $args{file_path} ;
-    $logger->info( "Read cds data from $file_path");
+    my $file_path = $args{file_path};
+    $logger->info("Read cds data from $file_path");
 
-    $self->node->load( step => [ $args{io_handle}->getlines ] ) ;
-    return 1 ;
+    $self->node->load( step => [ $args{io_handle}->getlines ] );
+    return 1;
 }
 
 sub write_cds_file {
-    my $self = shift;
-    my %args = @_ ;
-    my $file_path = $args{file_path} ;
+    my $self      = shift;
+    my %args      = @_;
+    my $file_path = $args{file_path};
     $logger->info("Write cds data to $file_path");
 
-    my $dump = $self->node->dump_tree(skip_auto_write => 'cds_file', check => $args{check} ) ;
-    $args{io_handle}->print( $dump ) ;
-    return 1 ;
+    my $dump = $self->node->dump_tree( skip_auto_write => 'cds_file', check => $args{check} );
+    $args{io_handle}->print($dump);
+    return 1;
 }
 
 sub read_perl {
     my $self = shift;
-    my %args = @_ ;
+    my %args = @_;
 
-    my $file_path = $args{file_path} ;
+    my $file_path = $args{file_path};
     $logger->info("Read Perl data from $file_path");
 
     my $pdata = do $file_path || die "Cannot open $file_path:$!";
-    $self->node->load_data( $pdata ) ;
-    return 1 ;
+    $self->node->load_data($pdata);
+    return 1;
 }
 
 sub write_perl {
-    my $self = shift;
-    my %args = @_ ;
-    my $file_path = $args{file_path} ;
+    my $self      = shift;
+    my %args      = @_;
+    my $file_path = $args{file_path};
     $logger->info("Write perl data to $file_path");
 
-    my $p_data = $self->node->dump_as_data(skip_auto_write => 'perl_file', check => $args{check} ) ;
-    my $dumper = Data::Dumper->new([$p_data]) ;
-    $dumper->Terse(1) ;
+    my $p_data = $self->node->dump_as_data( skip_auto_write => 'perl_file', check => $args{check} );
+    my $dumper = Data::Dumper->new( [$p_data] );
+    $dumper->Terse(1);
 
-    $args{io_handle}->print( $dumper->Dump , ";\n");
-    return 1 ;
+    $args{io_handle}->print( $dumper->Dump, ";\n" );
+    return 1;
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -728,7 +768,7 @@ Config::Model::BackendMgr - Load configuration node on demand
 
 =head1 VERSION
 
-version 2.047
+version 2.061
 
 =head1 SYNOPSIS
 
@@ -1228,7 +1268,7 @@ You can force to use all backend to write the files by specifying
 C<< backend => 'all' >>.
 
 You can force a specific config file to write with
-C<<config_file => 'foo/bar.conf' >>
+C<< config_file => 'foo/bar.conf' >>
 
 C<write_back> will croak if no write call-back are known for this node.
 
@@ -1247,7 +1287,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2013 by Dominique Dumont.
+This software is Copyright (c) 2014 by Dominique Dumont.
 
 This is free software, licensed under:
 
