@@ -1,72 +1,66 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2013 by Dominique Dumont.
+# This software is Copyright (c) 2014 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Report;
-{
-  $Config::Model::Report::VERSION = '2.047';
-}
+$Config::Model::Report::VERSION = '2.061';
 use Carp;
 use strict;
-use warnings ;
-
-use Config::Model::Exception ;
-use Config::Model::ObjTreeScanner ;
-use Text::Wrap ;
-
+use warnings;
 
+use Config::Model::Exception;
+use Config::Model::ObjTreeScanner;
+use Text::Wrap;
 
 sub new {
-    bless {}, shift ;
+    bless {}, shift;
 }
 
-
 sub report {
     my $self = shift;
 
-    my %args = @_;
+    my %args  = @_;
     my $audit = delete $args{audit} || 0;
-    my $node = delete $args{node} 
-      || croak "dump_tree: missing 'node' parameter";
+    my $node  = delete $args{node}
+        || croak "dump_tree: missing 'node' parameter";
 
     my $std_cb = sub {
         my ( $scanner, $data_r, $obj, $element, $index, $value_obj ) = @_;
 
-	# if element is a collection, get the value pointed by $index
-	$value_obj = $obj->fetch_element($element)->fetch_with_id($index) 
-	  if defined $index ;
+        # if element is a collection, get the value pointed by $index
+        $value_obj = $obj->fetch_element($element)->fetch_with_id($index)
+            if defined $index;
 
-	# get value or only customized value
-	my $value = $audit ? $value_obj->fetch_custom : $value_obj->fetch ;
+        # get value or only customized value
+        my $value = $audit ? $value_obj->fetch_custom : $value_obj->fetch;
 
         $value = '"' . $value . '"' if defined $value and $value =~ /\s/;
 
-	if (defined $value) {
-	    my $name = defined $index ? " $element:$index" : $element;
-	    push @$data_r , $obj->location." $name = $value";
-	    my $desc = $obj->get_help($element) ;
-	    if (defined $desc and $desc) {
-		push @$data_r , wrap ("\t","\t\t", "DESCRIPTION: $desc" ) ;
-	    }
-	    my $effect = $value_obj->get_help($value) ;
-	    if (defined $effect and $effect) {
-		push @$data_r, wrap ("\t","\t\t", "SELECTED: $effect" ) ;
-	    }
-	    push @$data_r , '' ; # to get empty line in report
-	}
+        if ( defined $value ) {
+            my $name = defined $index ? " $element:$index" : $element;
+            push @$data_r, $obj->location . " $name = $value";
+            my $desc = $obj->get_help($element);
+            if ( defined $desc and $desc ) {
+                push @$data_r, wrap( "\t", "\t\t", "DESCRIPTION: $desc" );
+            }
+            my $effect = $value_obj->get_help($value);
+            if ( defined $effect and $effect ) {
+                push @$data_r, wrap( "\t", "\t\t", "SELECTED: $effect" );
+            }
+            push @$data_r, '';    # to get empty line in report
+        }
     };
 
     my @scan_args = (
-		     experience  => delete $args{experience} || 'master',
-		     fallback    => 'all',
-		     auto_vivify => 0,
-		     leaf_cb     => $std_cb,
-		    );
+        fallback => 'all',
+        auto_vivify => 0,
+        leaf_cb     => $std_cb,
+    );
 
     my @left = keys %args;
     croak "Report: unknown parameter:@left" if @left;
@@ -74,10 +68,10 @@ sub report {
     # perform the scan
     my $view_scanner = Config::Model::ObjTreeScanner->new(@scan_args);
 
-    my @ret ;
-    $view_scanner->scan_node(\@ret ,$node);
+    my @ret;
+    $view_scanner->scan_node( \@ret, $node );
 
-    return join ("\n", @ret);
+    return join( "\n", @ret );
 }
 
 1;
@@ -96,7 +90,7 @@ Config::Model::Report - Reports data from config tree
 
 =head1 VERSION
 
-version 2.047
+version 2.061
 
 =head1 SYNOPSIS
 
@@ -229,7 +223,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2013 by Dominique Dumont.
+This software is Copyright (c) 2014 by Dominique Dumont.
 
 This is free software, licensed under:
 
