@@ -1,36 +1,31 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2013 by Dominique Dumont.
+# This software is Copyright (c) 2014 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Warper ;
-{
-  $Config::Model::Warper::VERSION = '2.047';
-}
-
-use Mouse ;
-use namespace::autoclean;
+package Config::Model::Warper;
+$Config::Model::Warper::VERSION = '2.061';
+use Mouse;
 
 use Log::Log4perl qw(get_logger :levels);
-use Data::Dumper ;
-use Storable qw/dclone/ ;
-use Config::Model::Exception ;
+use Data::Dumper;
+use Storable qw/dclone/;
+use Config::Model::Exception;
 use Carp;
 
+has 'follow' => ( is => 'ro', isa => 'HashRef[Str]', required => 1 );
+has 'rules'  => ( is => 'ro', isa => 'ArrayRef',     required => 1 );
 
-has 'follow' => ( is => 'ro', isa => 'HashRef[Str]' , required => 1 );
-has 'rules'  => ( is => 'ro', isa => 'ArrayRef' , required => 1 );
-
-has 'warped_object'  => ( 
-    is => 'ro', 
-    isa => 'Config::Model::AnyThing' , 
-    handles => [ 'notify_change' ],
-    weak_ref => 1, 
-    required => 1 
+has 'warped_object' => (
+    is       => 'ro',
+    isa      => 'Config::Model::AnyThing',
+    handles  => ['notify_change'],
+    weak_ref => 1,
+    required => 1
 );
 
 has '_values' => (
@@ -45,58 +40,57 @@ has '_values' => (
     },
 );
 
-has _computed_masters => ( is => 'rw',  isa => 'HashRef' , init_arg => undef ) ;
+has _computed_masters => ( is => 'rw', isa => 'HashRef', init_arg => undef );
 
 has [qw/_warped_nodes _registered_values/] =>
-  ( is => 'rw', isa => 'HashRef', init_arg => undef, default => sub { {} }, );
+    ( is => 'rw', isa => 'HashRef', init_arg => undef, default => sub { {} }, );
 
-has allowed => ( is => 'rw',  isa => 'ArrayRef' ) ;
-has morph => ( is => 'ro', isa => 'Bool' ) ;
+has allowed => ( is => 'rw', isa => 'ArrayRef' );
+has morph   => ( is => 'ro', isa => 'Bool' );
 
-my $logger = get_logger("Warper") ;
+my $logger = get_logger("Warper");
 
 # create the object, check args, but don't do anything else
 sub BUILD {
-    my $self = shift ;
-    
-    $logger->debug( "Warper new: created for ".$self->name);
-    $self->check_warp_args ;
+    my $self = shift;
 
-    $self->register_to_all_warp_masters ;
-    $self->refresh_values_from_master ;
-    $self->do_warp ;
+    $logger->debug( "Warper new: created for " . $self->name );
+    $self->check_warp_args;
+
+    $self->register_to_all_warp_masters;
+    $self->refresh_values_from_master;
+    $self->do_warp;
 }
 
 # should be called only at startup
 sub register_to_all_warp_masters {
-    my $self = shift ;
-    
+    my $self = shift;
+
     my $follow = $self->follow;
 
     # now, follow is only { w1 => 'warp1', w2 => 'warp2'}
-    foreach my $warper_name (keys %$follow) {
-        $self->register_to_one_warp_master ($warper_name);
+    foreach my $warper_name ( keys %$follow ) {
+        $self->register_to_one_warp_master($warper_name);
     }
-    
-}    
+
+}
 
 sub register_to_one_warp_master {
-    my $self        = shift;
+    my $self = shift;
     my $warper_name = shift || die "register_to_one_warp_master: missing warper_name";
 
     my $follow      = $self->follow;
     my $warper_path = $follow->{$warper_name};
-    $logger->debug( "Warper register_to_one_warp_master: ",
-        $self->name, " follows $warper_name" );
+    $logger->debug( "Warper register_to_one_warp_master: ", $self->name, " follows $warper_name" );
 
     # need to register also to all warped_nodes found on the path
     my @command = ($warper_path);
     my $warper;
-    my $warped_node ;
-    my $obj = $self->warped_object;
-    my $reg_values = $self->_registered_values ;
+    my $warped_node;
+    my $obj        = $self->warped_object;
+    my $reg_values = $self->_registered_values;
 
-    return if defined $reg_values->{$warper_name} ;
+    return if defined $reg_values->{$warper_name};
 
     while (@command) {
 
@@ -108,30 +102,32 @@ sub register_to_one_warp_master {
         );
 
         if ( not defined $obj ) {
-            $logger->debug( "Warper register_to_one_warp_master: aborted steps. Left '@command'" );
+            $logger->debug("Warper register_to_one_warp_master: aborted steps. Left '@command'");
             last;
         }
 
-        my $obj_loc = $obj->location ;
+        my $obj_loc = $obj->location;
 
-        $logger->debug( "Warper register_to_one_warp_master: step to master $obj_loc");
+        $logger->debug("Warper register_to_one_warp_master: step to master $obj_loc");
 
         if ( $obj->isa('Config::Model::Value') ) {
             $warper = $obj;
-            if (defined $warped_node) {
+            if ( defined $warped_node ) {
+
                 # keep obj ref to be able to unregister later on
-                $self->_warped_nodes->{$warped_node}{$warper_name} = $obj  ;
+                $self->_warped_nodes->{$warped_node}{$warper_name} = $obj;
             }
             last;
         }
 
         if ( $obj->isa('Config::Model::WarpedNode') ) {
-            $logger->debug( "Warper register_to_one_warp_master: register to warped_node $obj_loc");
-             if (defined $warped_node) {
+            $logger->debug("Warper register_to_one_warp_master: register to warped_node $obj_loc");
+            if ( defined $warped_node ) {
+
                 # keep obj ref to be able to unregister later on
-                $self->_warped_nodes->{$warped_node}{$warper_name} = $obj  ;
+                $self->_warped_nodes->{$warped_node}{$warper_name} = $obj;
             }
-            $warped_node = $obj_loc ;
+            $warped_node = $obj_loc;
             $obj->register( $self, $warper_name );
         }
     }
@@ -139,7 +135,7 @@ sub register_to_one_warp_master {
     if ( defined $warper and scalar @command ) {
         Config::Model::Exception::Model->throw(
             object => $self->warped_object,
-            error => "Some steps are left (@command) from warper path $warper_path",
+            error  => "Some steps are left (@command) from warper path $warper_path",
         );
     }
 
@@ -147,16 +143,15 @@ sub register_to_one_warp_master {
         "Warper register_to_one_warp_master:",
         $self->name,
         " is warped by $warper_name => '$warper_path' location in tree is: '",
-        defined $warper ? $warper->name : 'unknown' ,
-        "'"
+        defined $warper ? $warper->name : 'unknown', "'"
     );
-    
-    return unless defined $warper ;
+
+    return unless defined $warper;
 
     Config::Model::Exception::Model->throw(
         object => $self->warped_object,
         error  => "warper $warper_name => '$warper_path' is not a leaf"
-    ) unless $warper->isa('Config::Model::Value') ;
+    ) unless $warper->isa('Config::Model::Value');
 
     # warp will register this value object in another value object
     # (the warper).  When the warper gets a new value, it will
@@ -164,8 +159,8 @@ sub register_to_one_warp_master {
     # user.
 
     my $type = $warper->register( $self, $warper_name );
-    
-    $reg_values->{$warper_name} = $warper ;
+
+    $reg_values->{$warper_name} = $warper;
 
     # store current warp master value
     if ( $type eq 'computed' ) {
@@ -174,36 +169,30 @@ sub register_to_one_warp_master {
 }
 
 sub refresh_affected_registrations {
-    my ($self, $warped_node_location) = @_ ;
-    
-    my $wnref = $self->_warped_nodes ;
+    my ( $self, $warped_node_location ) = @_;
+
+    my $wnref = $self->_warped_nodes;
+
+    $logger->debug( "Warper refresh_affected_registrations: called on",
+        $self->name, " from $warped_node_location'" );
 
-    $logger->debug(
-        "Warper refresh_affected_registrations: called on",
-        $self->name,
-        " from $warped_node_location'"
-    );
-   
     #return unless defined $wnref ;
-    
+
     # remove and unregister obj affected by this warped node
-    my $ref = delete $wnref->{$warped_node_location} ;
-    
-    foreach my $warper_name (keys %$ref) {
-        $logger->debug(
-            "Warper refresh_affected_registrations: ",
-            $self->name,
-            " unregisters from $warper_name'"
-        );
-        delete $self->_registered_values->{$warper_name} ;
-        $ref -> {$warper_name} -> unregister( $self->name ) ;
+    my $ref = delete $wnref->{$warped_node_location};
+
+    foreach my $warper_name ( keys %$ref ) {
+        $logger->debug( "Warper refresh_affected_registrations: ",
+            $self->name, " unregisters from $warper_name'" );
+        delete $self->_registered_values->{$warper_name};
+        $ref->{$warper_name}->unregister( $self->name );
     }
 
-    $self->register_to_all_warp_masters ;
+    $self->register_to_all_warp_masters;
+
     #map {  $self->register_to_one_warp_master($_) } keys %$ref;
 }
 
-
 # should be called only at startup
 sub refresh_values_from_master {
     my $self = shift;
@@ -217,36 +206,36 @@ sub refresh_values_from_master {
     # should try to get values only for unregister or computed warp masters
     foreach my $warper_name ( keys %$follow ) {
         my $warper_path = $follow->{$warper_name};
-        $logger->debug( "Warper trigger: ",
-            $self->name, " following $warper_name" );
+        $logger->debug( "Warper trigger: ", $self->name, " following $warper_name" );
 
-       # warper can itself be warped out (part of a warped out node).
-       # not just 'not available'.
+        # warper can itself be warped out (part of a warped out node).
+        # not just 'not available'.
 
         my $warper = $self->warped_object->grab(
             step => $warper_path,
             mode => 'loose',
         );
 
-        if ( defined $warper and $warper->get_type eq 'leaf') {
+        if ( defined $warper and $warper->get_type eq 'leaf' ) {
+
             # read the warp master values, so I can warp myself just
             # after.
-            my $warper_value = $warper->fetch('allow_undef') ;
+            my $warper_value = $warper->fetch('allow_undef');
             $logger->debug( "Warper: '$warper_name' value is: '"
-                . ( defined $warper_value ? $warper_value : '<undef>' ) 
-                . "'" );
+                    . ( defined $warper_value ? $warper_value : '<undef>' )
+                    . "'" );
             $self->_set_value( $warper_name => $warper_value );
         }
-		elsif (defined $warper) {
-			Config::Model::Exception::Model -> throw (
-				error => "warp error: warp 'follow' parameter "
-						  ."does not point to a leaf element",
-				object => $self->warped_object
-		       ) ;
-		}
+        elsif ( defined $warper ) {
+            Config::Model::Exception::Model->throw(
+                error => "warp error: warp 'follow' parameter "
+                    . "does not point to a leaf element",
+                object => $self->warped_object
+            );
+        }
         else {
             # consider that the warp master value is undef
-            $self->_set_value($warper_name,'');
+            $self->_set_value( $warper_name, '' );
             $logger->debug("Warper:  '$warper_name' is not available");
         }
     }
@@ -254,104 +243,100 @@ sub refresh_values_from_master {
 }
 
 sub name {
-    my $self = shift ;
-    return "Warper of ".$self->warped_object->name ;
+    my $self = shift;
+    return "Warper of " . $self->warped_object->name;
 }
 
 # And I'm going to warp them ...
-sub warp_them
-  {
-    my $self = shift ;
+sub warp_them {
+    my $self = shift;
 
     # retrieve current value if not provided
-    my $value = @_ ? $_[0] 
-              :      $self->fetch_no_check ;
+    my $value =
+          @_
+        ? $_[0]
+        : $self->fetch_no_check;
 
-    foreach my $ref ( @{$self->{warp_these_objects}})
-      {
-        my ($warped, $warp_index) = @$ref ;
-        next unless defined $warped ; # $warped is a weak ref and may vanish
+    foreach my $ref ( @{ $self->{warp_these_objects} } ) {
+        my ( $warped, $warp_index ) = @$ref;
+        next unless defined $warped;    # $warped is a weak ref and may vanish
 
         # pure warp of object
-        $logger->debug("Warper ",$self->name," warp_them: (value ", (defined $value ? $value : 'undefined'),
-		  ") warping '",$warped->name, "'" );
-        $warped->warp($value,$warp_index) ;
-      }
-  }
-
-
+        $logger->debug(
+            "Warper ", $self->name,
+            " warp_them: (value ",
+            ( defined $value ? $value : 'undefined' ),
+            ") warping '", $warped->name, "'"
+        );
+        $warped->warp( $value, $warp_index );
+    }
+}
 
 sub check_warp_args {
-    my $self = shift ;
+    my $self = shift;
 
     # check that rules element are array ref and store them for
     # error checking
-    my $rules_ref =  $self->rules ;
+    my $rules_ref = $self->rules;
 
-    my @rules 
-      = ref $rules_ref eq 'HASH'  ? %$rules_ref :
-	ref $rules_ref eq 'ARRAY' ? @$rules_ref :
-	  Config::Model::Exception::Model
-	      -> throw ( error => "warp error: warp 'rules' parameter "
-			          ."is not a ref ($rules_ref)",
-			 object => $self->warped_object
-		       ) ;
+    my @rules =
+          ref $rules_ref eq 'HASH'  ? %$rules_ref
+        : ref $rules_ref eq 'ARRAY' ? @$rules_ref
+        : Config::Model::Exception::Model->throw(
+        error  => "warp error: warp 'rules' parameter " . "is not a ref ($rules_ref)",
+        object => $self->warped_object
+        );
 
-    my $allowed = $self->allowed ;
+    my $allowed = $self->allowed;
 
-    for (my $r_idx = 0; $r_idx < $#rules; $r_idx += 2) {
-        my $key_set = $rules[$r_idx] ;
-        my @keys = ref($key_set) ? @$key_set : ($key_set) ;
+    for ( my $r_idx = 0 ; $r_idx < $#rules ; $r_idx += 2 ) {
+        my $key_set = $rules[$r_idx];
+        my @keys = ref($key_set) ? @$key_set : ($key_set);
 
-        my $v = $rules[$r_idx + 1] ;
-        Config::Model::Exception::Model
-	    -> throw (
-		      object => $self->warped_object,
-		      error => "rules value for @keys is not a hash ref ($v)"
-		     ) 
-	      unless ref($v) eq 'HASH' ;
+        my $v = $rules[ $r_idx + 1 ];
+        Config::Model::Exception::Model->throw(
+            object => $self->warped_object,
+            error  => "rules value for @keys is not a hash ref ($v)"
+        ) unless ref($v) eq 'HASH';
 
-        foreach my $pkey (keys %$v) {
-            Config::Model::Exception::Model
-		-> throw (
-			  object => $self->warped_object,
-			  error => "Warp rules error for '@keys': '$pkey' ".
-			  "parameter is not allowed, ".
-			  "expected '".join("' or '",@$allowed)."'"
-			 ) 
-		  unless grep( $pkey eq $_ , @$allowed) ;
-	}
+        foreach my $pkey ( keys %$v ) {
+            Config::Model::Exception::Model->throw(
+                object => $self->warped_object,
+                error  => "Warp rules error for '@keys': '$pkey' "
+                    . "parameter is not allowed, "
+                    . "expected '"
+                    . join( "' or '", @$allowed ) . "'"
+            ) unless grep( $pkey eq $_, @$allowed );
+        }
     }
 }
 
 sub _dclone_key {
-    return map { ref $_ ? [ @$_ ] : $_ } @_ ;
+    return map { ref $_ ? [@$_] : $_ } @_;
 }
 
-
-# Internal. This method will change element properties (like level and
-# experience) according to the warp effect.  For instance, if a warp
-# rule make a node no longer available in a model, its level must
-# change to 'hidden'
+# Internal. This method will change element properties (like level) according to the warp effect.
+# For instance, if a warp rule make a node no longer available in a model, its level must change to
+# 'hidden'
 sub set_parent_element_property {
     my ( $self, $arg_ref ) = @_;
 
-    my $warped_object = $self->warped_object ;
+    my $warped_object = $self->warped_object;
 
-    my @properties = qw/level experience/ ;
+    my @properties = qw/level/;
 
     if ( defined $warped_object->index_value ) {
-        $logger->debug("Warper set_parent_element_property: called on hash or list, aborted" );
-        return ;
+        $logger->debug("Warper set_parent_element_property: called on hash or list, aborted");
+        return;
     }
-    
-    my $parent = $warped_object->parent ;
-    my $elt_name = $warped_object->element_name ;
+
+    my $parent   = $warped_object->parent;
+    my $elt_name = $warped_object->element_name;
     foreach my $property_name (@properties) {
         my $v = $arg_ref->{$property_name};
         if ( defined $v ) {
-            $logger->debug("Warper set_parent_element_property: set '",
-                $parent->name," $elt_name' $property_name with $v" );
+            $logger->debug( "Warper set_parent_element_property: set '",
+                $parent->name, " $elt_name' $property_name with $v" );
             $parent->set_element_property(
                 property => $property_name,
                 element  => $elt_name,
@@ -361,10 +346,10 @@ sub set_parent_element_property {
         else {
 
             # reset ensures that property is reset to known state by default
-            $logger->debug("Warper set_parent_element_property: reset $property_name" );
+            $logger->debug("Warper set_parent_element_property: reset $property_name");
             $parent->reset_element_property(
                 property => $property_name,
-                element  => $elt_name ,
+                element  => $elt_name,
             );
         }
     }
@@ -374,144 +359,158 @@ sub set_parent_element_property {
 sub trigger {
     my $self = shift;
 
-    my %old_value_set  = %{ $self->_values } ;
+    my %old_value_set = %{ $self->_values };
 
     if (@_) {
-        my ($value,$warp_name) = @_ ;
-        $logger -> debug( "Warper: trigger called on ",$self->name, " with value '", 
-		   defined $value ? $value : '<undef>',
-		   "' name $warp_name");
-        $self->_set_value($warp_name => $value || '') ;
+        my ( $value, $warp_name ) = @_;
+        $logger->debug(
+            "Warper: trigger called on ",
+            $self->name,
+            " with value '",
+            defined $value ? $value : '<undef>',
+            "' name $warp_name"
+        );
+        $self->_set_value( $warp_name => $value || '' );
     }
 
-    # read warp master values that are computed 
-    my $cm = $self->_computed_masters ;
-    foreach my $name (keys %$cm ) {
-        $self->_set_value($name => $cm->{$name}->fetch) ;
+    # read warp master values that are computed
+    my $cm = $self->_computed_masters;
+    foreach my $name ( keys %$cm ) {
+        $self->_set_value( $name => $cm->{$name}->fetch );
     }
 
     # check if new values are different from old values
-    my $same = 1 ;
-    foreach my $name ($self->_value_keys) {
-        my $old = $old_value_set   {$name};
+    my $same = 1;
+    foreach my $name ( $self->_value_keys ) {
+        my $old = $old_value_set{$name};
         my $new = $self->_get_value($name);
-        $same = 0 if ( $old ? 1 : 0  xor $new ? 1 : 0 )
-	  or ($old and $new and $new ne $old) ;
+        $same = 0
+            if ( $old ? 1 : 0 xor $new ? 1 : 0 )
+            or ( $old and $new and $new ne $old );
     }
 
     if ($same) {
-	no warnings "uninitialized" ;
-        if ($logger->is_debug) {
-            $logger ->debug("Warper: warp skipped because no change in value set ",
-		  "(old: '",join("' '", %old_value_set),"' new: '",
-		  join("' '",%{ $self->_values() }),"')");
+        no warnings "uninitialized";
+        if ( $logger->is_debug ) {
+            $logger->debug(
+                "Warper: warp skipped because no change in value set ",
+                "(old: '", join( "' '", %old_value_set ),
+                "' new: '", join( "' '", %{ $self->_values() } ), "')"
+            );
         }
-        return ;
+        return;
     }
 
-    $self->do_warp ;
+    $self->do_warp;
 }
 
 # undef values are changed to '' so compute_bool no longer returns
 # undef. It returns either 1 or 0
 sub compute_bool {
-    my $self = shift ;
-    my $expr = shift ;
+    my $self = shift;
+    my $expr = shift;
 
-    $logger ->debug("Warper compute_bool: called for '$expr'") ;
+    $logger->debug("Warper compute_bool: called for '$expr'");
 
-#    my $warp_value_set = $self->_values   ;
-    $logger ->debug("Warper compute_bool: data:\n", 
-		    Data::Dumper->Dump([$self->_values],['data']));
+    #    my $warp_value_set = $self->_values   ;
+    $logger->debug( "Warper compute_bool: data:\n",
+        Data::Dumper->Dump( [ $self->_values ], ['data'] ) );
 
     $expr =~ s/&(\w+)/\$self->warped_object->$1/g;
 
-    my @init_code ;
-    foreach my $warper_name ($self->_value_keys) {
-	my $v = $self->_get_value($warper_name) ;
+    my @init_code;
+    foreach my $warper_name ( $self->_value_keys ) {
+        my $v = $self->_get_value($warper_name);
 
-	my $code_v = (defined $v and $v =~ m/^[\d\.]$/) ? "$v"
-	           :  defined $v                        ? "'$v'"
-	           :                                      'undef' ;
+        my $code_v =
+              ( defined $v and $v =~ m/^[\d\.]$/ ) ? "$v"
+            : defined $v ? "'$v'"
+            :              'undef';
 
-	push @init_code, "my \$$warper_name = $code_v ;" ;
+        push @init_code, "my \$$warper_name = $code_v ;";
     }
 
-    my $perl_code = join("\n",@init_code, $expr);
-    $logger ->debug("Warper compute_bool: eval code '$perl_code'") ;
+    my $perl_code = join( "\n", @init_code, $expr );
+    $logger->debug("Warper compute_bool: eval code '$perl_code'");
 
     my $ret;
     {
-	no warnings "uninitialized" ;
-	$ret = eval($perl_code) ;
+        no warnings "uninitialized";
+        $ret = eval($perl_code);
     }
 
     if ($@) {
-        Config::Model::Exception::Model
-	    -> throw (
-		      object => $self->warped_object ,
-		      error => "Warp boolean expression failed:\n$@"
-		      . "eval'ed code is: \n$perl_code"
-		     ) 
+        Config::Model::Exception::Model->throw(
+            object => $self->warped_object,
+            error  => "Warp boolean expression failed:\n$@" . "eval'ed code is: \n$perl_code"
+        );
     }
 
-    $logger->debug("compute_bool: eval result: ", ($ret ? 'true' : 'false'));
-    return $ret ;
+    $logger->debug( "compute_bool: eval result: ", ( $ret ? 'true' : 'false' ) );
+    return $ret;
 }
 
 sub do_warp {
-    my $self = shift ;
+    my $self = shift;
 
-    my $warp_value_set = $self->_values   ;
-    my $rules          = dclone ($self->rules)   ;
-    my %rule_hash = @$rules ;
+    my $warp_value_set = $self->_values;
+    my $rules          = dclone( $self->rules );
+    my %rule_hash      = @$rules;
 
     # try all boolean expression with warp_value_set to get the
     # correct rule
 
     my $found_rule = {};
-    my $found_bool ='' ; # this variable may be used later in error message
+    my $found_bool = '';    # this variable may be used later in error message
 
     foreach my $bool_expr (@$rules) {
-	next if ref($bool_expr) ; # it's a rule not a bool expr
-	my $res = $self -> compute_bool( $bool_expr );
-	next unless $res ;
-	$found_bool = $bool_expr ;
-	$found_rule = $rule_hash{$bool_expr} || {};
-	$logger->debug("do_warp found rule for '$bool_expr':\n", 
-		       Data::Dumper->Dump ([$found_rule],['found_rule']));
-	last;
+        next if ref($bool_expr);    # it's a rule not a bool expr
+        my $res = $self->compute_bool($bool_expr);
+        next unless $res;
+        $found_bool = $bool_expr;
+        $found_rule = $rule_hash{$bool_expr} || {};
+        $logger->debug(
+            "do_warp found rule for '$bool_expr':\n",
+            Data::Dumper->Dump( [$found_rule], ['found_rule'] ) );
+        last;
     }
 
-    if ($logger->is_info) {
-        my @warp_str = map { defined $_ ? $_ : 'undef' } keys %$warp_value_set ;
+    if ( $logger->is_info ) {
+        my @warp_str = map { defined $_ ? $_ : 'undef' } keys %$warp_value_set;
 
-        $logger->info("do_warp: warp called from '$found_bool' on '",$self->warped_object->name,
-		      "' with elements '",join("','",@warp_str),
-		      "', warp rule is ", (scalar %$found_rule ? "" : 'not ') , 
-		      "found");
+        $logger->info(
+            "do_warp: warp called from '$found_bool' on '",
+            $self->warped_object->name,
+            "' with elements '",
+            join( "','", @warp_str ),
+            "', warp rule is ",
+            ( scalar %$found_rule ? "" : 'not ' ),
+            "found"
+        );
     }
 
-    $logger->debug("do_warp: call set_parent_element_property on '",$self->name,"' with ",
-	Data::Dumper->Dump ([$found_rule],['found_rule']));
+    $logger->debug( "do_warp: call set_parent_element_property on '",
+        $self->name, "' with ", Data::Dumper->Dump( [$found_rule], ['found_rule'] ) );
 
-    $self->set_parent_element_property ( $found_rule ) ;
+    $self->set_parent_element_property($found_rule);
 
-    $logger->debug("do_warp: call set_properties on '",$self->warped_object->name,"' with ",
-	Data::Dumper->Dump ([$found_rule],['found_rule']));
-    eval { $self->warped_object->set_properties(%$found_rule) ; };
+    $logger->debug(
+        "do_warp: call set_properties on '",
+        $self->warped_object->name,
+        "' with ", Data::Dumper->Dump( [$found_rule], ['found_rule'] ) );
+    eval { $self->warped_object->set_properties(%$found_rule); };
 
     if ($@) {
-        my @warp_str = map { defined $_ ? $_ : 'undef' } keys %$warp_value_set ;
-	my $e = $@ ;
-	my $msg = ref $e ? $e->as_string : $e ;
-	Config::Model::Exception::Model
-	    -> throw (
-		      object => $self->warped_object,
-		      error => "Warp failed when following '" . join("','",@warp_str)
-		             . "' from \"$found_bool\". Check model rules:\n\t"
-		             . $msg
-		     ) ;
+        my @warp_str = map { defined $_ ? $_ : 'undef' } keys %$warp_value_set;
+        my $e        = $@;
+        my $msg      = ref $e ? $e->as_string : $e;
+        Config::Model::Exception::Model->throw(
+            object => $self->warped_object,
+            error  => "Warp failed when following '"
+                . join( "','", @warp_str )
+                . "' from \"$found_bool\". Check model rules:\n\t"
+                . $msg
+        );
     }
 }
 
@@ -523,67 +522,71 @@ sub do_warp {
 # must be warped out by setting a warp master value that has not rule
 # attached. This case is indicated when $want_remove is set to 1
 sub warp_error {
-    my ($self) = @_ ;
+    my ($self) = @_;
 
-    return '' unless defined $self->{warp} ;
-    my $follow = $self->{warp}{follow} ;
-    my @rules  = @{$self->{warp}{rules}} ;
+    return '' unless defined $self->{warp};
+    my $follow = $self->{warp}{follow};
+    my @rules  = @{ $self->{warp}{rules} };
 
-    # follow is either ['warp1','warp2',...] 
+    # follow is either ['warp1','warp2',...]
     # or { warp1 => {....} , ...} or 'warp'
-    my @warper_paths = ref($follow) eq 'ARRAY' ? @$follow 
-                     : ref($follow) eq 'HASH'  ? values %$follow
-                     :                           ($follow) ;
+    my @warper_paths =
+          ref($follow) eq 'ARRAY' ? @$follow
+        : ref($follow) eq 'HASH'  ? values %$follow
+        :                           ($follow);
 
-    my $str = "You may solve the problem by modifying ".
-      (@warper_paths>1 ? "one or more of ": '').
-        "the following configuration parameters:\n" ;
+    my $str =
+          "You may solve the problem by modifying "
+        . ( @warper_paths > 1 ? "one or more of " : '' )
+        . "the following configuration parameters:\n";
 
     my $expected_error = 'Config::Model::Exception::UnavailableElement';
 
-    foreach my $warper_path ( @warper_paths ) {
-	my $warper_value ;
-	my $warper ;
+    foreach my $warper_path (@warper_paths) {
+        my $warper_value;
+        my $warper;
 
-	# try 
-	eval {
-	    $warper = $self->get_warper_object($warper_path);
-	    $warper_value = $warper->fetch ;
-	};
+        # try
+        eval {
+            $warper       = $self->get_warper_object($warper_path);
+            $warper_value = $warper->fetch;
+        };
 
-	# catch
-	if ( my $e = Exception::Class->caught($expected_error) )
-        {
-	    $str .= "\t'$warper_path' which is unavailable\n" ;
-	    next ;
+        # catch
+        if ( my $e = Exception::Class->caught($expected_error) ) {
+            $str .= "\t'$warper_path' which is unavailable\n";
+            next;
         }
 
-        $warper_value = 'undef' unless defined $warper_value ;
+        $warper_value = 'undef' unless defined $warper_value;
 
-        my @choice = defined $warper->choice ? @{$warper->choice} :
-	  $warper->{value_type} eq 'boolean' ? (0,1) : () ;
+        my @choice =
+              defined $warper->choice ? @{ $warper->choice }
+            : $warper->{value_type} eq 'boolean' ? ( 0, 1 )
+            :                                      ();
 
-        my @try = sort grep { $_ ne $warper_value } @choice ;
+        my @try = sort grep { $_ ne $warper_value } @choice;
 
-        $str .= "\t'".$warper->location. "': Try " ;
+        $str .= "\t'" . $warper->location . "': Try ";
 
-        my $a = $warper->{value_type} =~ /^[aeiou]/ ? 'an' : 'a' ;
+        my $a = $warper->{value_type} =~ /^[aeiou]/ ? 'an' : 'a';
 
-        $str .= @try ? "'".join ("' or '",@try)."' instead of " :
-          "$a $warper->{value_type} value different from " ;
+        $str .=
+            @try
+            ? "'" . join( "' or '", @try ) . "' instead of "
+            : "$a $warper->{value_type} value different from ";
 
-        $str .= "'$warper_value'\n" ;
+        $str .= "'$warper_value'\n";
 
-        if (defined $warper->{compute}) {
-            $str .= "\n\tHowever, '".$warper->name. "' ". 
-              $warper->compute_info."\n" ;
-	}
+        if ( defined $warper->{compute} ) {
+            $str .= "\n\tHowever, '" . $warper->name . "' " . $warper->compute_info . "\n";
+        }
     }
 
-    $str .= "Warp parameters:\n". Data::Dumper->Dump([$self->{warp}],['warp'])
-      if $logger->is_debug ;
+    $str .= "Warp parameters:\n" . Data::Dumper->Dump( [ $self->{warp} ], ['warp'] )
+        if $logger->is_debug;
 
-    return $str ;
+    return $str;
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -604,7 +607,7 @@ Config::Model::Warper - Warp tree properties
 
 =head1 VERSION
 
-version 2.047
+version 2.061
 
 =head1 SYNOPSIS
 
@@ -799,7 +802,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2013 by Dominique Dumont.
+This software is Copyright (c) 2014 by Dominique Dumont.
 
 This is free software, licensed under:
 
